import sys
import requests
import re
import xbmc
import xbmcgui
import xbmcplugin
import xbmcaddon
import json
import base64
from urllib.parse import quote, urlencode, parse_qsl
# Funzioni importate
from login import check_credentials
import updater
from mpd_playback import play_mpd_content
from hls_playback import play_hls_content

# Informazioni sull'addon
addon = xbmcaddon.Addon()
addon_handle = int(sys.argv[1])
base_url = sys.argv[0]

def get_json_from_api(file_path):
    # Costruisci l'URL per la richiesta
    url = f'https://monflix.atwebpages.com/api_json.php?path={quote(file_path)}'
    
    try:
        # Effettua la richiesta GET
        response = requests.get(url)
        response.raise_for_status()  # Controlla se ci sono errori HTTP
        
        # Log della risposta ricevuta
        xbmc.log(f"Risposta ricevuta da {url}: {response.text}", xbmc.LOGDEBUG)
        
        # Controllo se il contenuto è JSON
        if response.headers.get('Content-Type') == 'application/json':
            response_json = response.json()  # Decodifica come JSON
            return response_json
        else:
            xbmc.log(f"Contenuto non JSON ricevuto da {url}: {response.text}", xbmc.LOGERROR)
            return None
            
    except requests.exceptions.HTTPError as e:
        if e.response.status_code == 404:
            xbmc.log(f"File non trovato: {file_path}. URL: {url}", xbmc.LOGERROR)
        else:
            xbmc.log(f"Errore HTTP nel recupero del file {file_path}: {e}", xbmc.LOGERROR)
    except requests.exceptions.RequestException as e:
        xbmc.log(f"Errore nel recupero del file {file_path}: {str(e)}", xbmc.LOGERROR)
    except ValueError as e:
        xbmc.log(f"Errore nella decodifica del JSON ricevuto da {file_path}: {str(e)}", xbmc.LOGERROR)

    return None

# Funzione per costruire un URL con parametri
def build_url(query):
    return f"{base_url}?{urlencode(query)}"

def save_credentials(username, password):
    # Salva le credenziali nel file settings.xml
    addon.setSetting('username', username)
    addon.setSetting('password', password)

def login():
    dialog = xbmcgui.Dialog()

    # Controlla se ci sono credenziali salvate
    saved_username = addon.getSetting('username')
    saved_password = addon.getSetting('password')

    if saved_username and saved_password:
        # Se esistono, usa le credenziali salvate
        if check_credentials(saved_username, saved_password):
            dialog.notification('Login', 'Login effettuato con successo', xbmcgui.NOTIFICATION_INFO, 3000)
            return True
        else:
            dialog.notification('Login', 'Credenziali scadute o non valide, reinserisci.', xbmcgui.NOTIFICATION_ERROR, 3000)

    # Se non ci sono credenziali salvate, chiedi all'utente di inserirle
    username = dialog.input('Inserisci Username')
    password = dialog.input('Inserisci Password', option=xbmcgui.ALPHANUM_HIDE_INPUT)
    
    if check_credentials(username, password):
        dialog.notification('Login', 'Login effettuato con successo', xbmcgui.NOTIFICATION_INFO, 3000)
        
        # Salva le credenziali nel file settings.xml
        save_credentials(username, password)

        return True
    else:
        dialog.notification('Login', 'Login fallito', xbmcgui.NOTIFICATION_ERROR, 3000)
        return False

# Funzione per mostrare il menu principale
def main_menu():
    updater.check_for_updates()
    if not login():
        return

    # Categorie principali
    categories = ["Live TV", "Film", "Serie TV"]
    for category in categories:
        list_item = xbmcgui.ListItem(label=category)
        url = build_url({'action': 'open_category', 'category': category})
        xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

    xbmcplugin.endOfDirectory(addon_handle)

# Funzione per aprire una categoria
def open_category(category):
    try:
        if category == "Film":
            subcategories = ["Animazione", "Azione", "Avventura","Commedia", "Fantascienza", "Guerra", "Horror", "Dramma", "Family", "Crime", "Story", "Mistery", "Romance", "Thriller", "Western"]
            for subcategory in subcategories:
                list_item = xbmcgui.ListItem(label=subcategory)
                url = build_url({'action': 'initCategory', 'subcategory': subcategory, 'start': 0, 'end': 30})
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

        elif category == "Serie TV":
            subcategories = ["Dramma", "Commedia", "Fantascienza"]
            for subcategory in subcategories:
                list_item = xbmcgui.ListItem(label=subcategory)
                url = build_url({'action': 'load_series', 'subcategory': subcategory})
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

        elif category == "Live TV":
            play_live_tv_channels()

        xbmcplugin.endOfDirectory(addon_handle)

    except Exception as e:
        xbmcgui.Dialog().notification('Errore', f"Errore caricamento categoria: {str(e)}", xbmcgui.NOTIFICATION_ERROR, 5000)
        xbmc.log(f"Exception in open_category: {str(e)}", xbmc.LOGERROR)

def play_live_tv_channels():
    try:    
        channels = load_json("json/tv/dask.json")
        if not channels:
            xbmcgui.Dialog().notification('Errore', 'Nessun canale trovato nel file JSON', xbmcgui.NOTIFICATION_ERROR, 3000)
            xbmc.log("Nessun canale trovato nel file JSON", xbmc.LOGERROR)
            return
        
        xbmc.log(f"Numero di canali caricati: {len(channels)}", xbmc.LOGINFO)
        
        for channel in channels:
                if 'staff_id' in channel and 'name' in channel:
                    list_item = xbmcgui.ListItem(label=channel['name'])
                    list_item.setInfo('video', {'title': channel['name']})
                    list_item.setProperty('IsPlayable', 'true')
                    if 'logo' in channel:
                        list_item.setArt({'thumb': channel['logo'], 'icon': channel['logo'], 'fanart': channel['logo']})
                    url = build_url({'action': 'play', 'content_type': 'live', 'id': channel['staff_id'], 'mpd_url': channel})
                    xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)

        xbmcplugin.endOfDirectory(addon_handle)
    
    except Exception as e:
        xbmc.log(f"Errore in play_live_tv_channels: {str(e)}", level=xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', 'Errore durante il caricamento dei Live', xbmcgui.NOTIFICATION_ERROR, 5000)

def play_film_channels():
    try:
        # Carica la lista dei film da un file o da una fonte di dati
        films = load_movies()  # Assicurati che questa funzione sia implementata correttamente
        if not films:
            xbmcgui.Dialog().notification('Errore', 'Nessun film trovato', xbmcgui.NOTIFICATION_ERROR, 3000)
            return

        for film in films:
            if 'staff_id' in film and 'name' in film and 'hls_url' in film:
                list_item = xbmcgui.ListItem(label=film['name'])
                list_item.setInfo('video', {'title': film['name']})
                list_item.setProperty('IsPlayable', 'true')
                if 'logo' in film:
                    list_item.setArt({'thumb': film['logo'], 'icon': film['logo'], 'fanart': film['logo']})

                # Codifica l'URL HLS per evitare problemi con caratteri speciali
                encoded_hls_url = quote(film['hls_url'])
                
                # Crea un dizionario con i parametri per build_url
                url_params = {
                    'action': 'play',
                    'content_type': 'film',
                    'id': film['staff_id'],
                    'hls_url': encoded_hls_url
                }
                
                url = build_url(url_params)
                
                xbmc.log(f"URL costruito per il film {film['name']}: {url}", level=xbmc.LOGDEBUG)

                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)

        xbmcplugin.endOfDirectory(addon_handle)
    except Exception as e:
        xbmc.log(f"Errore in play_film_channels: {str(e)}", level=xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', 'Errore durante il caricamento dei film', xbmcgui.NOTIFICATION_ERROR, 5000)

def get_info(title, content_type):
    tmdb_api_key = "ded5b7afc33c6be7e0eaefce452217b1"
    r = requests.get(f"https://api.themoviedb.org/3/search/{content_type}?api_key={tmdb_api_key}&language=it-IT&query={requests.utils.quote(title)}")
    if r  and r.json():
        return r.json()

def load_json(url):
    content = get_json_from_api(url)
    if content is None:
        xbmc.log(f"Impossibile caricare il file JSON da repository: {url}", xbmc.LOGERROR)
        return []

    # Se content è già una lista, restituiscila direttamente
    if isinstance(content, list):
        return content

    try:
        # Rimuovi eventuali BOM all'inizio del file
        content = content.lstrip('\ufeff')
        # Prova a decodificare il JSON
        data = json.loads(content)  # Questo ora dovrebbe funzionare solo se content è una stringa
        return data
    except json.JSONDecodeError as e:
        xbmc.log(f"Errore nella decodifica del file JSON: {url}. Errore: {str(e)}", xbmc.LOGERROR)
        xbmc.log(f"Contenuto problematico: {content[:200]}...", xbmc.LOGDEBUG)  # Log dei primi 200 caratteri per debug
        return []

def initCategory(subcategory):
    if subcategory == "Animazione":
        json_url = 'json/film/Animazione/Animazione_part_1.json'
        json_url = 'json/film/Animazione/Animazione_part_2.json'
        json_url = 'json/film/Animazione/Animazione_part_3.json'
        json_url = 'json/film/Animazione/Animazione_part_4.json'
        json_url = 'json/film/Animazione/Animazione_part_5.json'
        json_url = 'json/film/Animazione/Animazione_part_6.json'
    if subcategory == "Azione":
        json_url = 'json/film/Azione/Azione_part_1.json'
        json_url = 'json/film/Azione/Azione_part_2.json'
        json_url = 'json/film/Azione/Azione_part_3.json'
        json_url = 'json/film/Azione/Azione_part_4.json'
        json_url = 'json/film/Azione/Azione_part_5.json'
        json_url = 'json/film/Azione/Azione_part_6.json'
        json_url = 'json/film/Azione/Azione_part_7.json'
        json_url = 'json/film/Azione/Azione_part_8.json'
        json_url = 'json/film/Azione/Azione_part_9.json'
        json_url = 'json/film/Azione/Azione_part_10.json'
        json_url = 'json/film/Azione/Azione_part_11.json'
        json_url = 'json/film/Azione/Azione_part_12.json'
        json_url = 'json/film/Azione/Azione_part_13.json'
    elif subcategory == "Avventura":
        json_url = 'json/film/Avventura/Avventura_part_1.json'
        json_url = 'json/film/Avventura/Avventura_part_2.json'
        json_url = 'json/film/Avventura/Avventura_part_3.json'
        json_url = 'json/film/Avventura/Avventura_part_4.json'
        json_url = 'json/film/Avventura/Avventura_part_5.json'
        json_url = 'json/film/Avventura/Avventura_part_6.json'
        json_url = 'json/film/Avventura/Avventura_part_7.json'
        json_url = 'json/film/Avventura/Avventura_part_8.json'
        json_url = 'json/film/Avventura/Avventura_part_9.json'
        json_url = 'json/film/Avventura/Avventura_part_10.json'
        json_url = 'json/film/Avventura/Avventura_part_11.json'
        json_url = 'json/film/Avventura/Avventura_part_12.json'
        json_url = 'json/film/Avventura/Avventura_part_13.json'
    elif subcategory == "Commedia":
        json_url = 'json/film/Commedia/Commedia_part_1.json'
        json_url = 'json/film/Commedia/Commedia_part_2.json'
        json_url = 'json/film/Commedia/Commedia_part_3.json'
        json_url = 'json/film/Commedia/Commedia_part_4.json'
        json_url = 'json/film/Commedia/Commedia_part_5.json'
        json_url = 'json/film/Commedia/Commedia_part_6.json'
        json_url = 'json/film/Commedia/Commedia_part_7.json'
        json_url = 'json/film/Commedia/Commedia_part_8.json'
        json_url = 'json/film/Commedia/Commedia_part_9.json'
        json_url = 'json/film/Commedia/Commedia_part_10.json'
        json_url = 'json/film/Commedia/Commedia_part_11.json'
        json_url = 'json/film/Commedia/Commedia_part_12.json'
        json_url = 'json/film/Commedia/Commedia_part_13.json'
    elif subcategory == "Fantascienza":
        json_url = 'json/film/Fantascienza/Fantascienza_part_1.json'
        json_url = 'json/film/Fantascienza/Fantascienza_part_2.json'
        json_url = 'json/film/Fantascienza/Fantascienza_part_3.json'
        json_url = 'json/film/Fantascienza/Fantascienza_part_4.json'
        json_url = 'json/film/Fantascienza/Fantascienza_part_5.json'
        json_url = 'json/film/Fantascienza/Fantascienza_part_6.json'
        json_url = 'json/film/Fantascienza/Fantascienza_part_7.json'
        json_url = 'json/film/Fantascienza/Fantascienza_part_8.json'
        json_url = 'json/film/Fantascienza/Fantascienza_part_9.json'
        json_url = 'json/film/Fantascienza/Fantascienza_part_10.json'
    elif subcategory == "Guerra":
        json_url = 'json/film/Guerra/Guerra_part_1.json'
        json_url = 'json/film/Guerra/Guerra_part_2.json'
    elif subcategory == "Horror":
        json_url = 'json/film/Horror/Horror_part_1.json'
        json_url = 'json/film/Horror/Horror_part_2.json'
        json_url = 'json/film/Horror/Horror_part_3.json'
        json_url = 'json/film/Horror/Horror_part_4.json'
        json_url = 'json/film/Horror/Horror_part_5.json'
        json_url = 'json/film/Horror/Horror_part_6.json'
        json_url = 'json/film/Horror/Horror_part_7.json'
        json_url = 'json/film/Horror/Horror_part_8.json'
        json_url = 'json/film/Horror/Horror_part_9.json'
        json_url = 'json/film/Horror/Horror_part_10.json'
    elif subcategory == "Dramma":
        json_url = 'json/film/Dramma/Dramma_part_1.json'
        json_url = 'json/film/Dramma/Dramma_part_2.json'
        json_url = 'json/film/Dramma/Dramma_part_3.json'
        json_url = 'json/film/Dramma/Dramma_part_4.json'
        json_url = 'json/film/Dramma/Dramma_part_5.json'
        json_url = 'json/film/Dramma/Dramma_part_6.json'
        json_url = 'json/film/Dramma/Dramma_part_7.json'
        json_url = 'json/film/Dramma/Dramma_part_8.json'
        json_url = 'json/film/Dramma/Dramma_part_9.json'
        json_url = 'json/film/Dramma/Dramma_part_10.json'
        json_url = 'json/film/Dramma/Dramma_part_11.json'
        json_url = 'json/film/Dramma/Dramma_part_12.json'
        json_url = 'json/film/Dramma/Dramma_part_13.json'
    elif subcategory == "Family":
        json_url = 'json/film/Famiglia/Famiglia_part_1.json'
        json_url = 'json/film/Famiglia/Famiglia_part_2.json'
        json_url = 'json/film/Famiglia/Famiglia_part_3.json'
        json_url = 'json/film/Famiglia/Famiglia_part_4.json'
        json_url = 'json/film/Famiglia/Famiglia_part_5.json'
        json_url = 'json/film/Famiglia/Famiglia_part_6.json'
        json_url = 'json/film/Famiglia/Famiglia_part_7.json'
        json_url = 'json/film/Famiglia/Famiglia_part_8.json'
        json_url = 'json/film/Famiglia/Famiglia_part_9.json'
        json_url = 'json/film/Famiglia/Famiglia_part_10.json'
        json_url = 'json/film/Famiglia/Famiglia_part_11.json'
    elif subcategory == "Crime":
        json_url = 'json/film/Crime/Crime_part_1.json'
        json_url = 'json/film/Crime/Crime_part_2.json'
        json_url = 'json/film/Crime/Crime_part_3.json'
        json_url = 'json/film/Crime/Crime_part_4.json'
        json_url = 'json/film/Crime/Crime_part_5.json'
        json_url = 'json/film/Crime/Crime_part_6.json'
        json_url = 'json/film/Crime/Crime_part_7.json'
        json_url = 'json/film/Crime/Crime_part_8.json'
        json_url = 'json/film/Crime/Crime_part_9.json'
        json_url = 'json/film/Crime/Crime_part_10.json'
        json_url = 'json/film/Crime/Crime_part_11.json'
    elif subcategory == "Story":
        json_url = 'json/film/Storia/Storia_part_1.json'
        json_url = 'json/film/Storia/Storia_part_2.json'
        json_url = 'json/film/Storia/Storia_part_3.json'
        json_url = 'json/film/Storia/Storia_part_4.json'
    elif subcategory == "Mistery":
        json_url = 'json/film/Mistero/Mistero_part_1.json'
        json_url = 'json/film/Mistero/Mistero_part_2.json'
        json_url = 'json/film/Mistero/Mistero_part_3.json'
        json_url = 'json/film/Mistero/Mistero_part_4.json'
        json_url = 'json/film/Mistero/Mistero_part_5.json'
        json_url = 'json/film/Mistero/Mistero_part_6.json'
    elif subcategory == "Romance":
        json_url = 'json/film/Romance/Romance_part_1.json'
        json_url = 'json/film/Romance/Romance_part_2.json'
        json_url = 'json/film/Romance/Romance_part_3.json'
        json_url = 'json/film/Romance/Romance_part_4.json'
        json_url = 'json/film/Romance/Romance_part_5.json'
        json_url = 'json/film/Romance/Romance_part_6.json'
        json_url = 'json/film/Romance/Romance_part_7.json'
        json_url = 'json/film/Romance/Romance_part_8.json'
        json_url = 'json/film/Romance/Romance_part_9.json'
        json_url = 'json/film/Romance/Romance_part_10.json'
        json_url = 'json/film/Romance/Romance_part_11.json'
    elif subcategory == "Thriller":
        json_url = 'json/film/Thriller/Thriller_part_1.json'
        json_url = 'json/film/Thriller/Thriller_part_2.json'
        json_url = 'json/film/Thriller/Thriller_part_3.json'
        json_url = 'json/film/Thriller/Thriller_part_4.json'
        json_url = 'json/film/Thriller/Thriller_part_5.json'
        json_url = 'json/film/Thriller/Thriller_part_6.json'
        json_url = 'json/film/Thriller/Thriller_part_7.json'
        json_url = 'json/film/Thriller/Thriller_part_8.json'
        json_url = 'json/film/Thriller/Thriller_part_9.json'
        json_url = 'json/film/Thriller/Thriller_part_10.json'
        json_url = 'json/film/Thriller/Thriller_part_11.json'
        json_url = 'json/film/Thriller/Thriller_part_12.json'
        json_url = 'json/film/Thriller/Thriller_part_13.json'
    elif subcategory == "Western":
        json_url = 'json/film/Western/Westerm_part_1.json'

    load_movies(load_json(json_url), 0, 10)

def load_movies(channels, start, end):
    if not isinstance(channels, list):
        xbmc.log(f"Error: channels is not a list: {channels}", level=xbmc.LOGERROR)
        return

    xbmc.log(f"Channel: {channels[start:end]}", level=xbmc.LOGERROR)

    for movie in channels[start:end]:
        if not isinstance(movie, dict):
            continue

        title = movie.get('name', 'Titolo non disponibile')
        info_film = get_info(title, "movie")
        
        if info_film and len(info_film["results"]) > 0:
            movie_info = info_film["results"][0]
            
            poster_path = movie_info.get('poster_path', '')
            backdrop_path = movie_info.get('backdrop_path', '')
            overview = movie_info.get('overview', 'Descrizione non disponibile')

            list_item = xbmcgui.ListItem(label=title)
            list_item.setInfo('video', {
                'title': title,
                'plot': overview
            })

            list_item.setArt({
                'thumb': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{poster_path}" if poster_path else '',
                'fanart': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{backdrop_path}" if backdrop_path else ''
            })
            
            url = build_url({'action': 'play', 'content_type': 'movie', 'title': title, 'hls_url': movie})
            xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)
    
    if end < len(channels):
        _start = end
        _end = _start + 10 if _start + 10 < len(channels) else len(channels)
        
        next_page_url = build_url({
            'action': 'next_page',
            'start': _start,
            'end': _end,
            'channels': json.dumps(channels)  # Converti in JSON per passare i parametri
        })
        next_page_item = xbmcgui.ListItem(label='Pagina successiva >>')
        xbmcplugin.addDirectoryItem(handle=addon_handle, url=next_page_url, listitem=next_page_item, isFolder=True)

    xbmcplugin.endOfDirectory(addon_handle)

# Router per gestire le azioni
def router(paramstring):
    params = dict(parse_qsl(paramstring[1:]))
    if params:
        if params['action'] == 'open_category':
            open_category(params['category'])
        elif params['action'] == 'initCategory':
            initCategory(params['subcategory'])
        elif params['action'] == 'next_page':
            channels = json.loads(params['channels'])
            load_movies(channels, int(params['start']), int(params['end']))
        elif params['action'] == 'play':
            if "mpd_url" in params:
                play_mpd_content(params['mpd_url'], addon_handle)
            elif "hls_url" in params:
                play_hls_content(params['hls_url'], addon_handle)
    else:
        main_menu()

# Avvio dell'addon
if __name__ == '__main__':
    router(sys.argv[2])